<?xml version="1.0" encoding="UTF-8" ?>
<xsl:stylesheet version="1.0" xmlns="urn:WebWorks-Document-Schema"
                              xmlns:wwdoc="urn:WebWorks-Document-Schema"
                              xmlns:wwxta="urn:WebWorks-XmlToolAdapter-Schema"
                              xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                              xmlns:msxsl="urn:schemas-microsoft-com:xslt"
                              xmlns:wwmode="urn:WebWorks-Engine-Mode"
                              xmlns:wwexsldoc="urn:WebWorks-XSLT-Extension-Document"
                              xmlns:wwditaconfig="urn:WebWorks-Dita-Configuration-Schema"
                              xmlns:wwtype="urn:WebWorks-Type-Schema"
                              xmlns:wwstyles="urn:WebWorks-Styles-Schema"
                              xmlns:wwfilesystem="urn:WebWorks-XSLT-Extension-FileSystem"
                              xmlns:wwstring="urn:WebWorks-XSLT-Extension-StringUtilities"
                              xmlns:wwuri="urn:WebWorks-XSLT-Extension-URI"
                              exclude-result-prefixes="wwdoc wwxta xsl msxsl wwmode wwexsldoc wwditaconfig wwtype wwstyles wwfilesystem wwuri"
>
 <xsl:param name="ParameterDocumentPath" />
 <xsl:param name="ParameterOriginalDocumentPath" />
 <xsl:param name="ParameterOutputPath" />

 <xsl:variable name="GlobalDitaValPath" select="wwstring:Replace($ParameterOriginalDocumentPath, '.ditamap', '.ditaval')" />
 <xsl:variable name="GlobalDitaVal" select="wwexsldoc:LoadXMLWithoutResolver($GlobalDitaValPath)" />

 <xsl:key name="element-by-id" match="*" use="@id" />

 <xsl:include href="wwxmladapter:scripts/dita/xsl/batch-files.xsl" />
 <xsl:include href="wwxmladapter:scripts/dita/xsl/generate-xml/locales.xsl" />
 <xsl:include href="wwxmladapter:scripts/dita/xsl/generate-xml/util.xsl" />
 <xsl:include href="wwxmladapter:scripts/dita/xsl/generate-xml/link.xsl" />
 <xsl:include href="wwxmladapter:scripts/dita/xsl/generate-xml/styles.xsl" />
 <xsl:include href="wwxmladapter:scripts/dita/xsl/generate-xml/content.xsl" />
 <xsl:include href="wwxmladapter:scripts/dita/xsl/generate-xml/paragraph.xsl" />
 <xsl:include href="wwxmladapter:scripts/dita/xsl/generate-xml/character.xsl" />
 <xsl:include href="wwxmladapter:scripts/dita/xsl/generate-xml/graphic.xsl" />
 <xsl:include href="wwxmladapter:scripts/dita/xsl/generate-xml/table.xsl" />
 <xsl:include href="wwxmladapter:scripts/dita/xsl/generate-xml/table-dita.xsl" />
 <xsl:include href="wwxmladapter:scripts/dita/xsl/generate-xml/marker.xsl" />
 <xsl:include href="wwxmladapter:scripts/dita/xsl/generate-xml/index.xsl" />
 <xsl:include href="wwxmladapter:scripts/dita/xsl/generate-xml/keywords.xsl" />
 <xsl:include href="wwxmladapter:scripts/dita/xsl/generate-xml/othermeta.xsl" />
 <xsl:include href="wwxmladapter:scripts/dita/xsl/generate-xml/prodinfo.xsl" />


 <xsl:output method="xml" version="1.0" indent="yes" encoding="utf-8" />
 <xsl:namespace-alias stylesheet-prefix="wwdoc" result-prefix="#default" />


 <xsl:template match="/">
  <!-- Working paths -->
  <!--               -->
  <xsl:variable name="VarScriptPath" select="concat(wwfilesystem:GetTempFileName(), '.bat')" />
  <xsl:variable name="VarReducedStructurePath" select="concat($ParameterDocumentPath, '.rst')" />
  <xsl:variable name="VarRawWIFPath" select="concat($ParameterDocumentPath, '.rawwif')" />

  <!-- Strip non-essential structure -->
  <!--                               -->
  <xsl:call-template name="Write_XSLT_Transform_Batch_File">
   <xsl:with-param name="ParamBatchFilePath" select="$VarScriptPath" />
   <xsl:with-param name="ParamInputPath" select="$ParameterDocumentPath" />
   <xsl:with-param name="ParamStylesheetPath" select="wwuri:AsFilePath('wwxmladapter:scripts/dita/xsl/generate-xml/stage-reduce.xsl')" />
   <xsl:with-param name="ParamOutputPath" select="$VarReducedStructurePath" />
  </xsl:call-template>
  <xsl:variable name="VarReturnCodeReduce">
   <xsl:call-template name="Execute_Batch_File">
    <xsl:with-param name="ParamBatchFilePath" select="$VarScriptPath" />
   </xsl:call-template>
  </xsl:variable>

  <!-- Structure reduced? -->
  <!--                    -->
  <xsl:if test="number($VarReturnCodeReduce) = 0">
   <xsl:variable name="VarDocument" select="wwexsldoc:LoadXMLWithoutResolver($VarReducedStructurePath, true())" />

   <xsl:variable name="VarResultAsXML">
    <wwdoc:Document version="1.0">
     <!-- Styles -->
     <!--        -->
     <wwdoc:Styles>
      <xsl:call-template name="Document-Styles">
       <xsl:with-param name="ParamDocumentContext" select="$VarDocument" />
       <xsl:with-param name="ParamDocumentPath" select="$ParameterDocumentPath" />
      </xsl:call-template>
     </wwdoc:Styles>

     <!-- Content -->
     <!--         -->
     <wwdoc:Content>
      <xsl:apply-templates select="$VarDocument" mode="wwmode:content" />
     </wwdoc:Content>
    </wwdoc:Document>
   </xsl:variable>
   <xsl:variable name="VarResult" select="msxsl:node-set($VarResultAsXML)" />
   <xsl:variable name="VarWriteResult" select="wwexsldoc:Document($VarResult, $VarRawWIFPath, 'utf-8', 'xml', '1.0', 'yes')" />

   <!-- Filter empty paragraphs -->
   <!--                         -->
   <xsl:call-template name="Write_XSLT_Transform_Batch_File">
    <xsl:with-param name="ParamBatchFilePath" select="$VarScriptPath" />
    <xsl:with-param name="ParamInputPath" select="$VarRawWIFPath" />
    <xsl:with-param name="ParamStylesheetPath" select="wwuri:AsFilePath('wwxmladapter:scripts/dita/xsl/generate-xml/stage-cleanup.xsl')" />
    <xsl:with-param name="ParamOutputPath" select="$ParameterOutputPath" />
   </xsl:call-template>
   <xsl:variable name="VarReturnCodeCleanup">
    <xsl:call-template name="Execute_Batch_File">
     <xsl:with-param name="ParamBatchFilePath" select="$VarScriptPath" />
    </xsl:call-template>
   </xsl:variable>
  </xsl:if>

  <!-- Clean up -->
  <!--          -->
  <xsl:variable name="VarDeleteScript" select="wwfilesystem:DeleteFile($VarScriptPath)" />
  <xsl:variable name="VarDeleteReducedStructureFile" select="wwfilesystem:DeleteFile($VarReducedStructurePath)" />
  <xsl:variable name="VarDeleteRawWIFFile" select="wwfilesystem:DeleteFile($VarRawWIFPath)" />
 </xsl:template>
</xsl:stylesheet>
